/* MOUSE.C - simplified/customized version of mouselib.c */

/* FOR USE WITH 320X200 GRAPHICS SCREEN !!! */

#include <dos.h>
#include <stdio.h>
#include <graphics.h>
#include <conio.h>
#include <stdlib.h>
#include <dir.h>

/* voids and functions */

void init_mouse(void);
void call_mouse(void);
void show_mouse(void);
void hide_mouse(void);
int mouse_x(void);
int mouse_y(void);
void set_mouse_x(int x);
void set_mouse_y(int y);
int button_pressed(void);
int did_move(void);

void graphic_mouse(char x_hot_point,char y_hot_point,int *data_off);
void arrow_cursor(void);
void bug_cursor(void);

/* mouse data */

struct REGPACK reg;

#define MOUSE_INTR       0x33   /* interrupt */
#define LEFT_BUTTON      1      /* bit 0 */
#define RIGHT_BUTTON     2      /* bit 1 */

int arrow_data[] = {

	0x3FFF,0x1FFF,0x0FFF,0x07FF,0x03FF,0x01FF,0x00FF,0x00FF,
	0x0FFF,0x1FFF,0x3FFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,

	0x0000,0x4000,0x6000,0x7000,0x7800,0x7C00,0x7E00,0x7000,
	0x6000,0x4000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000 
};

int bug_data[] = {

	0xF7EF,0xF7EF,0xBA5D,0xDC3B,0xDC3B,0xE427,0xF81F,0xF00F,
	0xE007,0xD00B,0xD00B,0xB00D,0xF00F,0xE817,0xDC3B,0xDFFB,

	0x0000,0x0810,0x05A0,0x2184,0x0240,0x0990,0x0240,0x05A0,
	0x0240,0x0240,0x25A4,0x0660,0x0660,0x0180,0x2004,0x0000 
};

/* ********************************************************* 
	
	your main or initializing function could look like this: 
	
	
	initgraph          do this first...

	init_mouse         ...before this

										 set the desired pointer
	arrow_cursor       or...
	bug_cursor         or...
	your own cursor    (see end of file) 
	
	show_mouse         show it...

	your function/program 
	
	hide_mouse         ...hide it when done

	closegraph
	
********************************************************* */


void init_mouse(void)     /* INITIALISE MOUSE (AFTER INITGRAPH!) */
{
	 reg.r_ax=0;            /* detect genius mouse */
	 reg.r_bx=0;          

	 call_mouse();

	 if (reg.r_ax==0) {
		 
		 closegraph();
		 
		 printf("Unable to initialize mouse!\n\n");
		 printf("Press any key...\b");
		 
		 bdos(0x0c,0,0);      /* flush keyboard */
		 getch();
		 
		 exit(1);
	 }
}

void call_mouse(void)     /* EXECUTE INTERRUPT */
{
	 intr(MOUSE_INTR,&reg);
}

void show_mouse(void)     /* SHOW MOUSE */
{
	 reg.r_ax=1;

	 call_mouse();
}

void hide_mouse(void)     /* HIDE MOUSE */
{
	 reg.r_ax=2;

	 call_mouse();
}

int mouse_x(void)         /* GET MOUSE X POSITION */
{
	 reg.r_ax=3;

	 call_mouse();

	 return(reg.r_cx/2);    /* divided by two for 320x200 */
}

int mouse_y(void)         /* GET MOUSE Y POSITION */
{
	 reg.r_ax=3;

	 call_mouse();

	 return(reg.r_dx);
}

void set_mouse_x(int x)   /* SET MOUSE X POSITION */
{
	 reg.r_ax=4;
	 reg.r_cx=x*2;

	 call_mouse();
}

void set_mouse_y(int y)   /* SET MOUSE Y POSITION */
{
	 reg.r_ax=4;
	 reg.r_dx=y;

	 call_mouse();
}

int button_pressed(void)  /* WHICH BUTTON (IF ANY)? */
{
	 reg.r_ax=3;

	 call_mouse();

	 if ((reg.r_bx & 1)!=0) return(LEFT_BUTTON);
	 else if ((reg.r_bx & 2)!=0) return(RIGHT_BUTTON);
}

int did_move(void)        /* WAS THERE ANY MOVEMENT? */
{
	 int x,y;

	 reg.r_ax=11;

	 call_mouse();

	 x=reg.r_cx;
	 y=reg.r_dx;

	 if (x!=0 || y!=0) return(true);  
	 else return(false);
}




/* LOAD MOUSE POINTER DATA */


void graphic_mouse(char x_hot_point,char y_hot_point,int *data_off)
{
	 reg.r_ax=9;
	 reg.r_bx=x_hot_point;
	 reg.r_cx=y_hot_point;
	 reg.r_dx=FP_OFF(data_off);  /* offset mouse pointer data */  
	 reg.r_es=FP_SEG(data_off);  /* segment for offset */

	 call_mouse();
}

void arrow_cursor(void)
{
	 graphic_mouse(0,0,arrow_data);
}

void bug_cursor(void)
{
	 graphic_mouse(8,3,bug_data);
}

